@@ -53,13 +53,6 @@ void LLVMToSPIRVDbgTran::transDebugMetadata() {
   if (DIF.compile_unit_count() == 0)
     return;
 
-  if (isNonSemanticDebugInfo()) {
-    if (!BM->isAllowedToUseVersion(VersionNumber::SPIRV_1_6))
-      BM->addExtension(SPIRV::ExtensionID::SPV_KHR_non_semantic_info);
-    else
-      BM->setMinSPIRVVersion(VersionNumber::SPIRV_1_6);
-  }
-
   for (DICompileUnit *CU : DIF.compile_units()) {
     transDbgEntry(CU);
     for (DIImportedEntity *IE : CU->getImportedEntities())
@@ -289,6 +282,8 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgEntryImpl(const MDNode *MDN) {
   if (!MDN)
     return BM->addDebugInfo(SPIRVDebug::DebugInfoNone, getVoidTy(),
                             SPIRVWordVec());
+  if (isNonSemanticDebugInfo())
+    BM->addExtension(SPIRV::ExtensionID::SPV_KHR_non_semantic_info);
   if (const DINode *DIEntry = dyn_cast<DINode>(MDN)) {
     switch (DIEntry->getTag()) {
     // Types
@@ -493,8 +488,6 @@ SPIRVWord LLVMToSPIRVDbgTran::mapDebugFlags(DINode::DIFlags DFlags) {
   if (BM->getDebugInfoEIS() == SPIRVEIS_NonSemantic_Shader_DebugInfo_200)
     if (DFlags & DINode::FlagBitField)
       Flags |= SPIRVDebug::FlagBitField;
-  if (DFlags & DINode::FlagEnumClass)
-    Flags |= SPIRVDebug::FlagIsEnumClass;
   return Flags;
 }
 
@@ -778,7 +771,7 @@ LLVMToSPIRVDbgTran::transDbgArrayTypeDynamic(const DICompositeType *AT) {
 
 SPIRVEntry *LLVMToSPIRVDbgTran::transDbgSubrangeType(const DISubrange *ST) {
   using namespace SPIRVDebug::Operand::TypeSubrange;
-  SPIRVWordVec Ops(MinOperandCount);
+  SPIRVWordVec Ops(OperandCount);
   auto TransOperand = [&Ops, this, ST](int Idx) -> void {
     Metadata *RawNode = nullptr;
     switch (Idx) {
@@ -791,6 +784,9 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgSubrangeType(const DISubrange *ST) {
     case CountIdx:
       RawNode = ST->getRawCountNode();
       break;
+    case StrideIdx:
+      RawNode = ST->getRawStride();
+      break;
     }
     if (!RawNode) {
       Ops[Idx] = getDebugInfoNoneId();
@@ -818,17 +814,8 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgSubrangeType(const DISubrange *ST) {
                          : getDebugInfoNoneId();
     }
   };
-  for (int Idx = 0; Idx < MinOperandCount; ++Idx)
+  for (int Idx = CountIdx; Idx < OperandCount; ++Idx)
     TransOperand(Idx);
-  if (auto *RawNode = ST->getRawStride()) {
-    Ops.resize(MaxOperandCount);
-    if (auto *Node = dyn_cast<MDNode>(RawNode))
-      Ops[StrideIdx] = transDbgEntry(Node)->getId();
-    else
-      Ops[StrideIdx] =
-          SPIRVWriter->transValue(ST->getStride().get<ConstantInt *>(), nullptr)
-              ->getId();
-  }
   return BM->addDebugInfo(SPIRVDebug::TypeSubrange, getVoidTy(), Ops);
 }
 
@@ -1629,19 +1616,20 @@ LLVMToSPIRVDbgTran::transDbgImportedEntry(const DIImportedEntity *IE) {
   auto Tag = static_cast<dwarf::Tag>(IE->getTag());
   // FIXME: 'OpenCL/bugged' version is kept because it's hard to remove it
   // It's W/A for missing 2nd index in OpenCL's implementation
-  const SPIRVWord OffsetIdx = static_cast<int>(isNonSemanticDebugInfo());
-  SPIRVWordVec Ops(OpenCL::OperandCount - OffsetIdx);
-  Ops[OpenCL::NameIdx] = BM->getString(IE->getName().str())->getId();
-  Ops[OpenCL::TagIdx] = SPIRV::DbgImportedEntityMap::map(Tag);
-  Ops[OpenCL::SourceIdx - OffsetIdx] = getSource(IE->getFile())->getId();
-  Ops[OpenCL::EntityIdx - OffsetIdx] = transDbgEntry(IE->getEntity())->getId();
-  Ops[OpenCL::LineIdx - OffsetIdx] = IE->getLine();
+  const SPIRVWord OffsetIdx =
+      isNonSemanticDebugInfo() ? OperandCount - NonSemantic::OperandCount : 0;
+  SPIRVWordVec Ops(OperandCount - OffsetIdx);
+  Ops[NameIdx] = BM->getString(IE->getName().str())->getId();
+  Ops[TagIdx] = SPIRV::DbgImportedEntityMap::map(Tag);
+  Ops[SourceIdx - OffsetIdx] = getSource(IE->getFile())->getId();
+  Ops[EntityIdx - OffsetIdx] = transDbgEntry(IE->getEntity())->getId();
+  Ops[LineIdx - OffsetIdx] = IE->getLine();
   // This version of DIImportedEntity has no column number
-  Ops[OpenCL::ColumnIdx - OffsetIdx] = 0;
-  Ops[OpenCL::ParentIdx - OffsetIdx] = getScope(IE->getScope())->getId();
+  Ops[ColumnIdx - OffsetIdx] = 0;
+  Ops[ParentIdx - OffsetIdx] = getScope(IE->getScope())->getId();
   if (isNonSemanticDebugInfo())
-    transformToConstant(Ops, {OpenCL::TagIdx, OpenCL::LineIdx - OffsetIdx,
-                              OpenCL::ColumnIdx - OffsetIdx});
+    transformToConstant(Ops,
+                        {TagIdx, LineIdx - OffsetIdx, ColumnIdx - OffsetIdx});
   return BM->addDebugInfo(SPIRVDebug::ImportedEntity, getVoidTy(), Ops);
 }
 
