@@ -396,12 +396,11 @@ namespace TypeVector = TypeArray;
 
 namespace TypeSubrange {
 enum {
-  LowerBoundIdx   = 0,
-  UpperBoundIdx   = 1,
-  CountIdx        = 2,
+  CountIdx        = 0,
+  LowerBoundIdx   = 1,
+  UpperBoundIdx   = 2,
   StrideIdx       = 3,
-  MinOperandCount = 3,
-  MaxOperandCount = 4
+  OperandCount    = 4
 };
 }
 
@@ -943,7 +942,7 @@ static std::map<ExpressionOpCode, unsigned> OpCountMap {
 }
 
 namespace ImportedEntity {
-namespace OpenCL {
+inline namespace OpenCL {
 // it's bugged version, note 2nd index is missing
 // FIXME: need to remove it after some graceful period
 enum {
@@ -987,9 +986,9 @@ enum {
 
 // helper function to get parent scope of debug instruction, to be used
 // to determine with which compile unit the particular instruction relates
-inline bool hasDbgInstParentScopeIdx(
-    const uint32_t Kind, uint32_t &ParentScopeIdx,
-    const SPIRV::SPIRVExtInstSetKind ExtKind = SPIRV::SPIRVEIS_OpenCL) {
+inline bool hasDbgInstParentScopeIdx(const uint32_t Kind,
+                                     uint32_t &ParentScopeIdx,
+                                     const SPIRV::SPIRVExtInstSetKind ExtKind = SPIRV::SPIRVEIS_OpenCL) {
   switch (Kind) {
   case SPIRVDebug::Typedef:
     ParentScopeIdx = Typedef::ParentIdx;
@@ -1034,10 +1033,7 @@ inline bool hasDbgInstParentScopeIdx(
     ParentScopeIdx = LocalVariable::ParentIdx;
     return true;
   case SPIRVDebug::ImportedEntity:
-    if (ExtKind == SPIRV::SPIRVEIS_OpenCL_DebugInfo_100)
-      ParentScopeIdx = ImportedEntity::OpenCL::ParentIdx;
-    else
-      ParentScopeIdx = ImportedEntity::NonSemantic::ParentIdx;
+    ParentScopeIdx = ImportedEntity::ParentIdx;
     return true;
   case SPIRVDebug::ModuleINTEL:
     ParentScopeIdx = ModuleINTEL::ParentIdx;
