@@ -175,7 +175,7 @@ void SPIRVEntry::encodeLine(spv_ostream &O) const {
   if (!Module)
     return;
   const std::shared_ptr<const SPIRVLine> &CurrLine = Module->getCurrentLine();
-  if (Line && ((CurrLine && *Line != *CurrLine) || !CurrLine)) {
+  if (Line && (!CurrLine || *Line != *CurrLine)) {
     O << *Line;
     Module->setCurrentLine(Line);
   }
@@ -183,8 +183,43 @@ void SPIRVEntry::encodeLine(spv_ostream &O) const {
     Module->setCurrentLine(nullptr);
 }
 
+namespace {
+bool isDebugLineEqual(const SPIRVExtInst &DL1, const SPIRVExtInst &DL2) {
+  std::vector<SPIRVWord> DL1Args = DL1.getArguments();
+  std::vector<SPIRVWord> DL2Args = DL2.getArguments();
+
+  using namespace SPIRVDebug::Operand::DebugLine;
+  assert(DL1Args.size() == OperandCount && DL2Args.size() == OperandCount &&
+         "Invalid number of operands");
+  return DL1Args[SourceIdx] == DL2Args[SourceIdx] &&
+         DL1Args[StartIdx] == DL2Args[StartIdx] &&
+         DL1Args[EndIdx] == DL2Args[EndIdx] &&
+         DL1Args[ColumnStartIdx] == DL2Args[ColumnStartIdx] &&
+         DL1Args[ColumnEndIdx] == DL2Args[ColumnEndIdx];
+}
+} // namespace
+
+void SPIRVEntry::encodeDebugLine(spv_ostream &O) const {
+  if (!Module)
+    return;
+  const std::shared_ptr<const SPIRVExtInst> &CurrDebugLine =
+      Module->getCurrentDebugLine();
+  if (DebugLine &&
+      (!CurrDebugLine || !isDebugLineEqual(*DebugLine, *CurrDebugLine))) {
+    O << *DebugLine;
+    Module->setCurrentDebugLine(DebugLine);
+  }
+  if (isEndOfBlock() ||
+      isExtInst(SPIRVEIS_NonSemantic_Shader_DebugInfo_100,
+                SPIRVDebug::DebugNoLine) ||
+      isExtInst(SPIRVEIS_NonSemantic_Shader_DebugInfo_200,
+                SPIRVDebug::DebugNoLine))
+    Module->setCurrentDebugLine(nullptr);
+}
+
 void SPIRVEntry::encodeAll(spv_ostream &O) const {
   encodeLine(O);
+  encodeDebugLine(O);
   encodeWordCountOpCode(O);
   encode(O);
   encodeChildren(O);
@@ -301,6 +336,11 @@ void SPIRVEntry::setLine(const std::shared_ptr<const SPIRVLine> &L) {
   SPIRVDBG(if (L) spvdbgs() << "[setLine] " << *L << '\n';)
 }
 
+void SPIRVEntry::setDebugLine(const std::shared_ptr<const SPIRVExtInst> &DL) {
+  DebugLine = DL;
+  SPIRVDBG(if (DL) spvdbgs() << "[setDebugLine] " << *DL << '\n';)
+}
+
 void SPIRVEntry::addMemberDecorate(SPIRVMemberDecorate *Dec) {
   assert(canHaveMemberDecorates() &&
          MemberDecorates.find(Dec->getPair()) == MemberDecorates.end());
@@ -541,7 +581,7 @@ SPIRVEntryPoint::SPIRVEntryPoint(SPIRVModule *TheModule,
                                  SPIRVExecutionModelKind TheExecModel,
                                  SPIRVId TheId, const std::string &TheName,
                                  std::vector<SPIRVId> Variables)
-    : SPIRVAnnotation(TheModule->get<SPIRVFunction>(TheId),
+    : SPIRVAnnotation(OpEntryPoint, TheModule->get<SPIRVFunction>(TheId),
                       getSizeInWords(TheName) + Variables.size() + 3),
       ExecModel(TheExecModel), Name(TheName), Variables(Variables) {}
 
@@ -588,6 +628,9 @@ void SPIRVExecutionMode::decode(std::istream &I) {
   case ExecutionModeNumSIMDWorkitemsINTEL:
   case ExecutionModeSchedulerTargetFmaxMhzINTEL:
   case internal::ExecutionModeStreamingInterfaceINTEL:
+  case internal::ExecutionModeMaximumRegistersINTEL:
+  case internal::ExecutionModeMaximumRegistersIdINTEL:
+  case internal::ExecutionModeNamedMaximumRegistersINTEL:
     WordLiterals.resize(1);
     break;
   default:
@@ -609,7 +652,8 @@ SPIRVForward *SPIRVAnnotationGeneric::getOrCreateTarget() const {
 }
 
 SPIRVName::SPIRVName(const SPIRVEntry *TheTarget, const std::string &TheStr)
-    : SPIRVAnnotation(TheTarget, getSizeInWords(TheStr) + 2), Str(TheStr) {}
+    : SPIRVAnnotation(OpName, TheTarget, getSizeInWords(TheStr) + 2),
+      Str(TheStr) {}
 
 void SPIRVName::encode(spv_ostream &O) const { getEncoder(O) << Target << Str; }
 
@@ -631,8 +675,6 @@ void SPIRVLine::encode(spv_ostream &O) const {
 
 void SPIRVLine::decode(std::istream &I) {
   getDecoder(I) >> FileName >> Line >> Column;
-  std::shared_ptr<const SPIRVLine> L(this);
-  Module->setCurrentLine(L);
 }
 
 void SPIRVLine::validate() const {
