@@ -368,7 +368,6 @@ public:
   void takeDecorates(SPIRVEntry *);
   void takeDecorateIds(SPIRVEntry *);
   void takeMemberDecorates(SPIRVEntry *);
-  void replaceTargetIdInDecorates(SPIRVId);
 
   /// After a SPIRV entry is created during reading SPIRV binary by default
   /// constructor, this function is called to allow the SPIRV entry to resize
@@ -696,9 +695,9 @@ public:
 
   llvm::Optional<ExtensionID> getRequiredExtension() const override {
     switch (static_cast<unsigned>(ExecMode)) {
-    case ExecutionModeMaximumRegistersINTEL:
-    case ExecutionModeMaximumRegistersIdINTEL:
-    case ExecutionModeNamedMaximumRegistersINTEL:
+    case internal::ExecutionModeMaximumRegistersINTEL:
+    case internal::ExecutionModeMaximumRegistersIdINTEL:
+    case internal::ExecutionModeNamedMaximumRegistersINTEL:
       return ExtensionID::SPV_INTEL_maximum_registers;
     default:
       return {};
@@ -767,9 +766,9 @@ public:
              IsOtherFP(EMK);
     };
     auto IsMaxRegisters = [&](auto EMK) {
-      return EMK == ExecutionModeMaximumRegistersINTEL ||
-             EMK == ExecutionModeMaximumRegistersIdINTEL ||
-             EMK == ExecutionModeNamedMaximumRegistersINTEL;
+      return EMK == internal::ExecutionModeMaximumRegistersINTEL ||
+             EMK == internal::ExecutionModeMaximumRegistersIdINTEL ||
+             EMK == internal::ExecutionModeNamedMaximumRegistersINTEL;
     };
     auto IsCompatible = [&](SPIRVExecutionMode *EM0, SPIRVExecutionMode *EM1) {
       if (EM0->getTargetId() != EM1->getTargetId())
@@ -801,12 +800,6 @@ public:
       return nullptr;
     return Loc->second;
   }
-  SPIRVExecutionModeId *getExecutionModeId(SPIRVExecutionModeKind EMK) const {
-    auto Loc = ExecModes.find(EMK);
-    if (Loc == ExecModes.end())
-      return nullptr;
-    return static_cast<SPIRVExecutionModeId *>(Loc->second);
-  }
   SPIRVExecutionModeRange
   getExecutionModeRange(SPIRVExecutionModeKind EMK) const {
     return ExecModes.equal_range(EMK);
@@ -904,11 +897,8 @@ public:
     case CapabilityVectorComputeINTEL:
     case CapabilityVectorAnyINTEL:
       return ExtensionID::SPV_INTEL_vector_compute;
-    case CapabilityFunctionVariantsINTEL:
-    case CapabilitySpecConditionalINTEL:
-      return ExtensionID::SPV_INTEL_function_variants;
-    case internal::CapabilityBFloat16ArithmeticINTEL:
-      return ExtensionID::SPV_INTEL_bfloat16_arithmetic;
+    case internal::CapabilityFastCompositeINTEL:
+      return ExtensionID::SPV_INTEL_fast_composite;
     default:
       return {};
     }
@@ -1058,6 +1048,8 @@ _SPIRV_OP(ImageDrefGather)
 _SPIRV_OP(QuantizeToF16)
 _SPIRV_OP(ArrayLength)
 _SPIRV_OP(OuterProduct)
+_SPIRV_OP(SMulExtended)
+_SPIRV_OP(UMulExtended)
 _SPIRV_OP(DPdx)
 _SPIRV_OP(DPdy)
 _SPIRV_OP(Fwidth)
@@ -1090,6 +1082,7 @@ _SPIRV_OP(NamedBarrierInitialize)
 _SPIRV_OP(MemoryNamedBarrier)
 _SPIRV_OP(GetKernelMaxNumSubgroups)
 _SPIRV_OP(GetKernelLocalSizeForSubgroupCount)
+_SPIRV_OP(SizeOf)
 #undef _SPIRV_OP
 
 } // namespace SPIRV
