@@ -96,12 +96,14 @@ public:
   bool isTypeStruct() const;
   bool isTypeVector() const;
   bool isTypeJointMatrixINTEL() const;
+  bool isTypeCooperativeMatrixKHR() const;
   bool isTypeVectorInt() const;
   bool isTypeVectorFloat() const;
   bool isTypeVectorBool() const;
   bool isTypeVectorOrScalarInt() const;
   bool isTypeVectorOrScalarFloat() const;
   bool isTypeVectorOrScalarBool() const;
+  bool isTypeVectorPointer() const;
   bool isTypeSubgroupAvcINTEL() const;
   bool isTypeSubgroupAvcMceINTEL() const;
 };
@@ -1089,5 +1091,33 @@ public:
   SPIRVValue *getUse() const { return Args.size() > 4 ? Args[4] : nullptr; }
 };
 
+class SPIRVTypeCooperativeMatrixKHR : public SPIRVType {
+  SPIRVType *CompType;
+  std::vector<SPIRVValue *> Args;
+
+public:
+  const static Op OC = OpTypeCooperativeMatrixKHR;
+  const static SPIRVWord FixedWC = 7;
+  // Incomplete constructor
+  SPIRVTypeCooperativeMatrixKHR(SPIRVModule *M, SPIRVId TheId,
+                                SPIRVType *CompType,
+                                std::vector<SPIRVValue *> Args);
+  // Incomplete constructor
+  SPIRVTypeCooperativeMatrixKHR();
+  _SPIRV_DCL_ENCDEC
+  llvm::Optional<ExtensionID> getRequiredExtension() const override {
+    return ExtensionID::SPV_KHR_cooperative_matrix;
+  }
+  SPIRVCapVec getRequiredCapability() const override {
+    return getVec(CapabilityCooperativeMatrixKHR);
+  }
+
+  SPIRVType *getCompType() const { return CompType; }
+  SPIRVValue *getScope() const { return Args[0]; }
+  SPIRVValue *getRows() const { return Args[1]; }
+  SPIRVValue *getColumns() const { return Args[2]; }
+  SPIRVValue *getUse() const { return Args[3]; }
+};
+
 } // namespace SPIRV
 #endif // SPIRV_LIBSPIRV_SPIRVTYPE_H
